package com.jay.monitor.data.client.utils;

import com.jay.monitor.data.client.ext.MqMonitorRuleCallback;
import com.jay.monitor.data.core.enums.MQInvokeType;
import com.jay.monitor.data.core.enums.MQTypeEnum;
import com.jay.monitor.data.core.model.serializable.MQDataDTO;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;

/**
 * @Module 工具
 * @Description 将一些通用的监控数据方法进行抽取
 * @Author liukaixiong
 * @Date 2020/10/30 17:02
 */
public class MonitorUtils {

    public static MQDataDTO getProducerMqDataDTO(String topic, String key, Integer partitionIndex, Long offset, String value, Integer status) {
        MQDataDTO mq = new MQDataDTO();
        mq.setTopic(topic);
        mq.setMsgId(offset + "");
        mq.setCreateDate(new Date());
        mq.setMqType(MQTypeEnum.kafka.name());
        mq.setPartitionName(partitionIndex + "");
        mq.setJsonContent(value);
        mq.setGroupKeyName(key);
        mq.setStatus(status);
        mq.setInvokeType(MQInvokeType.producer);
        return mq;
    }

    public static MQDataDTO getProducerMqDataDTO(Map<String, BiConsumer<String, MQDataDTO>> ruleMap, String topic, String key, Integer partitionIndex, Long offset, String value, Integer status) {
        MQDataDTO mqDataDTO = getProducerMqDataDTO(topic, key, partitionIndex, offset, value, status);
        mqDataDTO.setInvokeType(MQInvokeType.producer);
        MonitorUtils.applyData(ruleMap, topic, value, mqDataDTO);
        return mqDataDTO;
    }

    public static MQDataDTO getConsumerMqDataDTO(Map<String, BiConsumer<String, MQDataDTO>> ruleMap, String topic, String key, Integer partitionIndex, Long offset, String value, Integer status) {
        MQDataDTO mqDataDTO = getProducerMqDataDTO(topic, key, partitionIndex, offset, value, status);
        mqDataDTO.setInvokeType(MQInvokeType.consumer);
        MonitorUtils.applyData(ruleMap, topic, value, mqDataDTO);
        return mqDataDTO;
    }

    /**
     * 留给客户端一个回调的扣子，让开发者根据自己的关注的数据去设置
     *
     * @param ruleMap
     * @param topic
     * @param value
     * @param mqDataDTO
     */
    public static void applyData(Map<String, BiConsumer<String, MQDataDTO>> ruleMap, String topic, String value, MQDataDTO mqDataDTO) {
        if (ruleMap != null) {
            // 默认通用的数据处理
            BiConsumer<String, MQDataDTO> consumer = ruleMap.get("DEFAULT_ALL");
            if (consumer != null) {
                consumer.accept(value, mqDataDTO);
            }

            consumer = ruleMap.get(topic);
            if (consumer != null) {
                consumer.accept(value, mqDataDTO);
            }
        }
    }

    /**
     * 获取监控回调的具体执行规则
     *
     * @param monitorRuleCallbackList 执行规则
     * @return
     */
    public static Map<String, BiConsumer<String, MQDataDTO>> getMqMonitorCallback(List<MqMonitorRuleCallback> monitorRuleCallbackList) {
        if (monitorRuleCallbackList != null && monitorRuleCallbackList.size() > 0) {
            Map<String, BiConsumer<String, MQDataDTO>> ruleMap = new HashMap<>();
            for (int i = 0; i < monitorRuleCallbackList.size(); i++) {
                MqMonitorRuleCallback monitorRuleCallback = monitorRuleCallbackList.get(i);
                ruleMap.putAll(monitorRuleCallback.getRuleMap());
            }
            return ruleMap;
        }
        return null;
    }
}
