package com.jay.monitor.data.server.store.mysql;

import com.cat.file.message.MessageManagerProcess;
import com.cat.file.message.internal.DefaultMessageTree;
import com.cat.file.message.internal.MessageTree;
import com.jay.monitor.data.core.enums.MsgTypeEnums;
import com.jay.monitor.data.core.model.serializable.MQDataDTO;
import com.jay.monitor.data.server.daos.JayMonitorMqMapperExt;
import com.jay.monitor.data.server.enums.StoreType;
import com.jay.monitor.data.server.models.entity.JayMonitorMq;
import com.jay.monitor.data.server.store.AbstractStoreProcess;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.LongAdder;

/**
 * @Module 数据存储器
 * @Description Mysql的MQ存储实现
 * @Author liukaixiong
 * @Date 2020/10/29 13:36
 */
@Component
@Lazy
public class MysqlMQStoreProcess extends AbstractStoreProcess<MQDataDTO> {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private LongAdder counter = new LongAdder();

    /**
     * 队列的大小
     */
    public int defaultSize = 5000;

    @Autowired
    @Qualifier("jayMonitorMqMapperExt")
    private JayMonitorMqMapperExt mqMapper;

    @Autowired
    private MessageManagerProcess messageManagerProcess;

    @Override
    public StoreType storeType() {
        return StoreType.MYSQL;
    }

    @Override
    public MsgTypeEnums msgTypeEnums() {
        return MsgTypeEnums.MQ;
    }

    @Override
    public String tableName() {
        return "jay_monitor_mq";
    }

    /**
     * 加速消费,单条记录插入会造成大量的并发写
     *
     * @param dataList
     */
    @Override
    public void speedUpConsumerList(List<MQDataDTO> dataList) {
        List<JayMonitorMq> monitorList = new ArrayList<>();

        for (int i = 0; i < dataList.size(); i++) {
            MQDataDTO request = dataList.get(i);
            JayMonitorMq mqData = builderJayMonitorMq(request);
            monitorList.add(mqData);
        }

        mqMapper.batchPartitionInsert(getPartitionId(), monitorList);

        for (int i = 0; i < monitorList.size(); i++) {
            Integer id = monitorList.get(i).getId();
            MQDataDTO dataDTO = dataList.get(i);
            MessageTree messageTree = new DefaultMessageTree(dataDTO.getApplicationName(), dataDTO.getIp(), id, dataDTO.getCreateDate(), dataDTO.getJsonContent());
            this.messageManagerProcess.insert(messageTree);
            logger.debug("添加成功:" + messageTree.getMessageId());
        }
    }

    @Override
    public String searchContentById(String date, String id) throws Exception {
        JayMonitorMq jayMonitorMq = mqMapper.selectPartitionById(date, Integer.valueOf(id));
        MessageTree messageTree = new DefaultMessageTree(jayMonitorMq.getApplicationName(), jayMonitorMq.getApplicationIp(), jayMonitorMq.getId(), jayMonitorMq.getCreatedTime(), null);
        String messageId = messageTree.getMessageId();
        return this.messageManagerProcess.getMessage(messageId);
    }

    private JayMonitorMq builderJayMonitorMq(MQDataDTO request) {
        JayMonitorMq mqData = new JayMonitorMq();
        try {
            BeanUtils.copyProperties(request, mqData);
            mqData.setApplicationName(request.getApplicationName());
            mqData.setApplicationIp(request.getIp());
            mqData.setCreatedTime(request.getCreateDate());
            mqData.setDataGroupName(request.getGroupName());
            mqData.setDataGroupKey(request.getGroupKeyName());
            mqData.setDataId(request.getDataId());
            mqData.setLogId(request.getLogId());
            mqData.setInvokeType(request.getInvokeType().name());
        } catch (Exception e) {
            logger.error("builderJayMonitorMq", e);
        }
        return mqData;
    }


}
